unit Main;
interface
uses Windows,Messages,SysUtils,Classes,Graphics,Controls,Forms,Dialogs,
     StdCtrls,Registry,ExtCtrls,MPlayer,savInit;
type TSaverBasics=class(TThread)
                    private
                      FForceRect:TRect;
                      procedure SetForceRect(ARect:TRect);
                    protected
                      Canvas:TCanvas; //the canvas to draw the saver action to
                      Left,           //dimensions of the display window
                      Top,            //all drawing must be done inside
                      Width,          //the here specified dimensions
                      Height,SleepTime:Integer;
                      procedure AStep; virtual;
                      //override this and implement a step in your screen saver,
                      //this gets called again and again until the saver finishes
                    public
                      constructor Create(ACanvas:TCanvas; DrawRect:TRect); virtual;
                      //ACanvas holds the canvas to draw the saver action to and the
                      //DrawRect holds the wanted rectangle, usually this is the full
                      //canvas
                      procedure Execute; override;
                      //don't care about this
                      procedure ClearForceRect;
                      //call this if you have redrawn the ForceRect to set it to 0,0,0,0
                      property ForceRect:TRect read FForceRect write SetForceRect;
                      {this holds a ForceRect and gets called whenever a rectangle needs
                       painting. Caused by a password dialog dragged over the window or
                       windows being dragged over the preview window. Make sure you draw
                       the ForceRect in your AStep function to make your saver look nice}
                    end;

procedure ExecPreview;
procedure ExecSetPwd;

type TSaverThread=class(TSaverBasics)
                      Religion,Fade,NrofFilms,R,G,B,
                      EndValueR,EndValueG,EndValueB:Integer;
                      Hypno,BlankScreen,StepVideo,FileOK,NT,
                      IncColourR,IncColourG,IncColourB,Down:Boolean;
                      AviPanel:TPanel;
                      AviPlay:TMediaPlayer;
                      constructor Create(ACanvas:TCanvas; DrawRect:TRect); override;
                      procedure AStep; override;
                      procedure ScrRepaint;
                      function RandomRGB(var CurrentValue:Integer; var IncColour:Boolean;
                               var EndValue:Integer):Integer;
                      procedure Delphi;
                      procedure DDxPreview;
                      procedure SetNT;
                      procedure CreateAviPlay;
                      procedure AviPlayNudge;
                      procedure NewVideo;
                      procedure DelphiDx;
                    end;

type TD97=class(TForm)
    MouseTimer:TTimer;
    Min2Fade:TTimer;
    procedure FormCreate(Sender:TObject);
    procedure FormShow(Sender:TObject);
    procedure FormPaint(Sender:TObject);
    procedure FormMouseMove(Sender:TObject; Shift:TShiftState; X,
      Y:Integer);
    procedure FormMouseUp(Sender:TObject; Button:TMouseButton;
      Shift:TShiftState; X, Y:Integer);
    procedure FormMouseDown(Sender:TObject; Button:TMouseButton;
      Shift:TShiftState; X, Y:Integer);
    procedure MouseTimerTimer(Sender:TObject);
    procedure Min2FadeTimer(Sender:TObject);
    procedure FormClose(Sender:TObject; var Action:TCloseAction);
  private
    MyThread:TSaverThread;
    procedure CreateParams (var Params:TCreateParams); override;
    procedure WMKeyDown (var Msg:TWMKeyDown); message WM_KEYDOWN;
    procedure WMSysKeyDown (var Msg:TWMSysKeyDown); message WM_SYSKEYDOWN;
    procedure WMEraseBkgnd (var Msg:TMessage); message WM_ERASEBKGND;
    procedure CloseIfOk;
    procedure CloseSaver;
    procedure MouseEvent;
  public
      IgnoreMouse:Integer;
  end;

var D97:TD97;
    NrMin,Chosen:Integer;
    FirstRun,PreviewMode,Password:Boolean;
    AviPlayReg:TRegistry;
implementation
{$R *.DFM}
{$R DDx.res} //Include DDx.bmp, adds 1kb to exe
const TicksToIgnore=4;//Number of mouseevents to ignore before closing screensaver

procedure TSaverBasics.SetForceRect(ARect:TRect);
begin //Area to repaint, always enlarge, never shrink
  if FForceRect.Left>ARect.Left then
    FForceRect.Left:=ARect.Left;
  if FForceRect.Top>ARect.Top then
    FForceRect.Top:=ARect.Top;
  if FForceRect.Right<ARect.Right then
    FForceRect.Right:=ARect.Right;
  if FForceRect.Bottom<ARect.Bottom then
    FForceRect.Bottom:=ARect.Bottom;
end;

procedure TSaverBasics.AStep;
begin //Class implemented in descendant classes
end;

constructor TSaverBasics.Create(ACanvas:TCanvas; DrawRect:TRect);
begin //set local variables
  Canvas:=ACanvas;
  Left:=DrawRect.Left;
  Top:=DrawRect.Top;
  Width:=DrawRect.Right-DrawRect.Left;
  Height:=DrawRect.Bottom-DrawRect.Top;
  FForceRect:=DrawRect;
  //initialize the thread as starting immediately
  inherited Create(False);
  //and now initialize some thread properties to the appropriate values
  FreeOnTerminate:=True;
end;

procedure TSaverBasics.Execute;
begin
  If Assigned(Canvas) then
  begin //do nothing if there is no canvas to draw to
        While not Terminated do
        begin
              Synchronize(AStep);
              If SleepTime>0 then //Send thread to sleep for delay/sleeptime ms
                 Sleep(SleepTime);
        end;
  end;
end;

procedure TSaverBasics.ClearForceRect;
begin
  FForceRect:=Rect(0,0,0,0);
end;

constructor TSaverThread.Create(ACanvas:TCanvas; DrawRect:TRect);
var Registry:TRegistry;
begin
  inherited Create(ACanvas,DrawRect);
  Registry:=TRegistry.Create;
  try
    Registry.RootKey:=HKEY_LOCAL_MACHINE;
    if Registry.OpenKey('Software\SIG Inc\Delphi 97 screensaver\1.0',False) then
    begin
         If Registry.ReadBool('Delphi97') then
            Chosen:=1 //Delphi Deluxe screensaver
         else
             Chosen:=0; //Delphi screensaver
         If Registry.ReadInteger('Religion')=1 then
            Religion:=1
         else
             If Registry.ReadInteger('Religion')=2 then
                Religion:=2
         else
             Religion:=0;
         If Registry.ReadBool('Hypnotic') then
            Hypno:=True
         else
             Hypno:=False;
         Fade:=Registry.ReadInteger('Fade');
         NrofFilms:=Registry.ReadInteger('Nr of chosen films');
         If Registry.ReadBool('Blank') then
            BlankScreen:=True
         else
             BlankScreen:=False;
         If Registry.ReadBool('Step') then
            StepVideo:=True
         else
             StepVideo:=False;
         SleepTime:=Registry.ReadInteger('Delay');
    end;
  except
        //Ignore everything
  end;
  Registry.CloseKey;
  Registry.Free;
end;

procedure TSaverThread.ScrRepaint;
begin //Repaint FForceRect area
      Canvas.Brush.Color:=clBlack;
      Canvas.Brush.Style:=bsSolid;
      Canvas.FillRect(FForceRect);
      ClearForceRect;
end;

procedure TSaverThread.AStep;
begin //Triggered every delay/sleeptime ms
     ScrRepaint;
     Case Chosen of
          0: Delphi;
          1: DelphiDx;
     end;
     FirstRun:=False;
end;

//Delphi screensaver start--------------------------------------------------
function TSaverThread.RandomRGB(var CurrentValue:Integer; var IncColour:Boolean;
         var EndValue:Integer):Integer;
begin //Generate a random colour cycle
     if IncColour then
        if CurrentValue<EndValue then
           Inc(CurrentValue)
        else
        begin
             if (Trunc(Random(2)))=0 then
                IncColour:=True
             else
                 IncColour:=False;
             EndValue:=Trunc(Random(255));
        end
        else
        begin
             if CurrentValue>EndValue then
                Dec(CurrentValue)
             else
             begin
                  if (Trunc(Random(2)))=0 then
                     IncColour:=True
                  else
                      IncColour:=False;
                  EndValue:=Trunc(Random(255));
             end;
        end;
     RandomRGB:=CurrentValue;
end;

procedure TSaverThread.Delphi;
var Column,PenWidth,TopofScreen,BottomofScreen,FourthofScreen:Integer;
begin //Paints chosen religion text across screen
     With Canvas do
     begin
          PenWidth:=Round(Height*0.05);
          TopofScreen:=Round(Height*0.05);
          BottomofScreen:=Round(Height*0.95);
          FourthofScreen:=Round(Height*0.25);
          if Hypno then
          begin
               If Down then
               begin
                    Pen.Width:=Pen.Width-1;
                    If Pen.Width=0 then
                       Down:=False;
               end
               else
               begin
                    Pen.Width:=Pen.Width+1;
                    if Pen.Width>PenWidth then
                       Down:=True;
               end;
          end
          else
              Pen.Width:=PenWidth;
          If (NrMin<Fade) or (Fade=0) then
             Pen.Color:=TColor(RGB(RandomRGB(R,IncColourR,EndValueR),
                RandomRGB(G,IncColourG,EndValueG),
                RandomRGB(B,IncColourB,EndValueB)))
          else
          begin
               If R>0 then
                  Dec(R);
               If G>0 then
                  Dec(G);
               If B>0 then
                  Dec(B);
               If (R=0) and (G=0) and (B=0) and (Pen.Width=PenWidth) then
                  NrMin:=0;
              Pen.Color:=TColor(RGB(R,G,B));
          end;
          Case Religion of
               0: Begin
                    Column:=Trunc(Width/12);
                    Polyline([Point(Column,BottomofScreen), //D
                      Point(Column,TopofScreen),
                      Point((Column*2),(Height div 2)),
                      Point(Column,BottomofScreen)]);
                    Polyline([Point((Column*4),TopofScreen), //E
                      Point((Column*3),(Height div 2)),
                      Point((Column*4),(Height div 2)),
                      Point((Column*3),(Height div 2)),
                      Point((Column*4),BottomofScreen)]);
                    Polyline([Point((Column*5),TopofScreen), //L
                      Point((Column*5),BottomofScreen),
                      Point((Column*6),BottomofScreen)]);
                    Polyline([Point((Column*7),TopofScreen), //P
                      Point((Column*7),BottomofScreen),
                      Point((Column*7),FourthofScreen),
                      Point((Column*8),FourthofScreen div 2),
                      Point((Column*7),TopofScreen)]);
                    Polyline([Point((Column*9),TopofScreen), //H
                      Point((Column*9),BottomofScreen),
                      Point((Column*9),Trunc(FourthofScreen*3)),
                      Point((Column*10),Trunc(FourthofScreen*3)),
                      Point((Column*10),TopofScreen),
                      Point((Column*10),BottomofScreen)]);
                    Polyline([Point(Trunc(Column*11),TopofScreen), //.
                      Point(Trunc(Column*11),FourthofScreen div 4)]);
                    Polyline([Point(Trunc(Column*11),FourthofScreen div 2), //I
                      Point(Trunc(Column*11),(BottomofScreen))]);
               end;
               1: Begin
                  Column:=Trunc(Width/11);
                  Polyline([Point(Column,BottomofScreen), //N
                    Point(Column,TopofScreen),
                    Point((Column*2),(BottomofScreen)),
                    Point(Column*2,TopofScreen)]);
                  Polyline([Point((Column*3),TopofScreen), //O
                    Point((Column*4),TopofScreen),
                    Point((Column*4),(BottomofScreen)),
                    Point((Column*3),(BottomofScreen)),
                    Point((Column*3),TopofScreen)]);
                  Polyline([Point((Column*5),TopofScreen), //T
                    Point((Column*6),TopofScreen),
                    Point(Trunc(Column*5.5),TopofScreen),
                    Point(Trunc(Column*5.5),BottomofScreen)]);
                  Polyline([Point((Column*8),TopofScreen), //E
                    Point((Column*7),TopofScreen),
                    Point((Column*7),(Height div 2)),
                    Point((Column*8),(Height div 2)),
                    Point((Column*7),(Height div 2)),
                    Point((Column*7),BottomofScreen),
                    Point((Column*8),BottomofScreen)]);
                  Polyline([Point((Column*10),FourthofScreen), //S
                    Point((Column*10),TopofScreen),
                    Point((Column*9),TopofScreen),
                    Point((Column*9),Height div 2),
                    Point((Column*10),height div 2),
                    Point((Column*10),BottomofScreen),
                    Point((Column*9),BottomofScreen),
                    Point((Column*9),Trunc(FourthofScreen*3))]);
               end;
               2: Begin
                  Column:=Trunc(Width/5);
                  Polyline([Point(Column,BottomofScreen), //N
                    Point(Column,TopofScreen),
                    Point((Column*2),BottomofScreen),
                    Point(Column*2,TopofScreen)]);
                  Polyline([Point((Column*3),TopofScreen), //T
                    Point((Column*4),TopofScreen),
                    Point(Trunc(Column*3.5),TopofScreen),
                    Point(Trunc(Column*3.5),BottomofScreen)]);
               end;
          end;
     end;
end; //Delphi screensaver end--------------------------------------------------

//Delphi Deluxe screensaver start----------------------------------------------
procedure TSaverThread.DDxPreview;
var DDxImage:TBitmap;
begin //Show DDx bitmap in preview window
      DDxImage:=TBitmap.Create;
      //DDxImage.LoadFromResourceName(hInstance,'DDx');
      DDxImage.Handle:=LoadBitmap(hInstance,'DDx');
      Canvas.Draw((Width div 2)-(DDxImage.Width div 2),
         (Height div 2)-(DDxImage.Height div 2),DDxImage);
      DDxImage.Free;
end;

procedure TSaverThread.SetNT;
var OSVer:TOSVersionInfo;
begin //Determine wether running under NT or not
      OSVer.dwOSVersionInfoSize:=SizeOf(OSVer);
      GetVersionEx(OSVer);
      If OSVer.dwPlatformID=VER_PLATFORM_WIN32_NT then
         NT:=True
      else
          NT:=False;
end;

procedure TSaverThread.CreateAviPlay;
var RandFilm:String;
begin //Create and initialize AviPlayReg, AviPanel and AviPlay objects
     AviPlayReg:=TRegistry.Create;
     AviPlayReg.RootKey:=HKEY_LOCAL_MACHINE;
     AviPlayReg.OpenKey('Software\SIG Inc\Delphi 97 screensaver\1.0',False);
     If NT then
     begin //getting round visual "bugs" in Win95/NT
          AviPanel:=TPanel.Create(D97);
          AviPanel.Parent:=D97;
          AviPanel.Visible:=False;
          AviPlay:=TMediaPlayer.Create(AviPanel);
          AviPlay.Parent:=AviPanel;
          Canvas.Brush.Color:=clBlack; //blank screen to hide generic AviPanel
          Canvas.Brush.Style:=bsSolid; //shown in upper left corner
          Canvas.FillRect(Rect(0,0,Width,Height));
     end
     else
     begin
          AviPlay:=TMediaPlayer.Create(D97);
          AviPlay.Parent:=D97;
     end;
     AviPlay.Visible:=False;
     AviPlay.Wait:=True;
     AviPlay.Top:=-100; //otherwise a rectangle appears in
     AviPlay.Left:=-100; //the avi's upper left corner, icon?
     AviPlay.AutoEnable:=False;
     AviPlay.VisibleButtons:=[];
     AviPlay.DeviceType:=dtAVIVideo;
     RandFilm:=IntToStr(Trunc(Random(NrofFilms))+1);
     If FileExists(AviPlayReg.ReadString('Film'+RandFilm)) then
     begin
          FileOK:=True;
          AviPlay.FileName:=AviPlayReg.ReadString('Film'+RandFilm);
          AviPlay.Display:=D97;
          AviPlay.Open; //get dimensions of the avi video
          AviPlay.DisplayRect:=Rect(0,0,0,0);
          AviPlayNudge;
          If NT then
             AviPanel.Visible:=True;
          AviPlay.Visible:=True;
          If StepVideo then
          begin
               AviPlay.TimeFormat:=tfFrames;
               AviPlay.Frames:=1; //if not set defaults to steps of 10% of avi length
               AviPlay.Step;
          end
          else
              AviPlay.Play;
     end
     else
         FileOK:=False;
end;

procedure TSaverThread.AviPlayNudge;
var  AviWidth,AviHeight,NewX,NewY:Integer;
begin //Make sure the avi is fully visible (if possible)
      AviWidth:=AviPlay.DisplayRect.Right-AviPlay.DisplayRect.Left;
      AviHeight:=AviPlay.DisplayRect.Bottom-AviPlay.DisplayRect.Top;
      If AviWidth>=Width then
         NewX:=0
      else
          NewX:=Trunc(Random((Width-AviWidth)));
      If AviHeight>=Height then
         NewY:=0
      else
          NewY:=Trunc(Random((Height-AviHeight)));
      AviPlay.Close;
      If NT then
      begin
           AviPanel.Left:=NewX;
           AviPanel.Top:=NewY;
           AviPanel.Width:=AviWidth;
           AviPanel.Height:=AviHeight;
           AviPlay.Display:=AviPanel;
      end
      else
          AviPlay.Display:=D97;
      AviPlay.Open;
      If not NT then
         AviPlay.DisplayRect:=Rect(NewX,NewY,0,0);
end;

procedure TSaverThread.NewVideo;
var RandFilm:String;
begin //Load new avi video
      If NT then
         AviPanel.Visible:=False;
      AviPlay.Visible:=False;
      If BlankScreen then
      begin
           Canvas.Brush.Color:=clBlack;
           Canvas.Brush.Style:=bsSolid;
           Canvas.FillRect(Rect(0,0,Width,Height));
      end;
      RandFilm:=IntToStr(Trunc(Random(NrofFilms))+1);
      If FileExists(AviPlayReg.ReadString('Film'+RandFilm)) then
      begin
           FileOK:=True;
           AviPlay.FileName:=AviPlayReg.ReadString('Film'+RandFilm);
           AviPlay.Display:=D97;
           AviPlay.Open;
           AviPlay.DisplayRect:=Rect(0,0,0,0);
           AviPlayNudge;
           If NT then
              AviPanel.Visible:=True;
           AviPlay.Visible:=True;
           If StepVideo then
           begin
                AviPlay.TimeFormat:=tfFrames;
                AviPlay.Frames:=1;
                AviPlay.Step;
           end
           else
               AviPlay.Play;
      end
      else
          FileOK:=False;
end;

procedure TSaverThread.DelphiDx;
begin //Play or step through avi's
      If not PreviewMode then
      begin
           If NrofFilms>0 then
           begin //if not any films are chosen, acts as a blank screen screensaver
                If FirstRun then
                begin
                     SetNT;
                     CreateAviPlay;
                     Exit;
                end;
                If FileOK then
                   If StepVideo then
                   begin
                        AviPlay.TimeFormat:=tfFrames;
                        AviPlay.Frames:=1;
                        If AviPlay.Position<AviPlay.TrackLength[1] then
                           AviPlay.Step
                        else
                            NewVideo;
                   end
                   else
                       If AviPlay.Position=AviPlay.TrackLength[1] then
                          NewVideo;
           end;
      end
      else
          DDxPreview;
end; //Delphi Deluxe screensaver end-------------------------------------------

var MyThread:TSaverThread;
    PrevRect:TRect;
//this is the window proc for the preview window, it only implements 3 messages
function MyWndProc(Wnd:HWnd; Msg:Integer; wParam:Word; lParam:Integer):Integer; far; stdcall;
begin //I don't know if it's ok to quit on WM_CLOSE, but I had no problems doing so
  if (Msg=WM_DESTROY) or (Msg=WM_CLOSE) then
    PostMessage(Wnd,WM_QUIT,0,0)
  else
  if Msg=WM_PAINT then //on paint is always the complete background now
    MyThread.ForceRect:=PrevRect;
  DefWindowProc(Wnd,Msg,wParam,lParam);
end;

procedure TD97.FormCreate(Sender:TObject);
var Dummy:Integer;
begin
  Top:=0;
  Left:=0;
  Width:=Screen.Width;
  Height:=Screen.Height;
  SystemParametersInfo(97,Word(True),@Dummy,0); //no alt-tab or ctrl-alt-del any more
  IgnoreMouse:=TicksToIgnore;
  ShowCursor(false); //hide mouse cursor
  Randomize;
  MouseTimer.Enabled:=True;
  Min2Fade.Enabled:=True;
end;

procedure TD97.FormShow(Sender:TObject);
begin //if the graphics is not yet running, create the thread
  if not Assigned(MyThread) then
    MyThread:=TSaverThread.Create(Canvas,Rect(0,0,Width,Height));
end;

procedure TD97.FormPaint(Sender:TObject);
begin //this gets called when the password change dialog is moved over
//the screen, so you should do something intelligent here and repaint
//the screen parts which must be repainted.
  If (Chosen=1) and (not Password) then
  else
      MyThread.ForceRect:=Canvas.ClipRect; //tell saver to repaint this rectangle
end;

procedure TD97.FormMouseMove(Sender:TObject; Shift:TShiftState; X,
  Y:Integer);
begin
  MouseEvent; //you will recieve at least one mouse move after startup
end;

procedure TD97.FormMouseUp(Sender:TObject; Button:TMouseButton;
  Shift:TShiftState; X, Y:Integer);
begin
  MouseEvent; //you may recieve one after startup
end;

procedure TD97.FormMouseDown(Sender:TObject; Button:TMouseButton;
  Shift:TShiftState; X, Y:Integer);
begin
  CloseIfOk; //mousedown is not sent to me accidently, always close
end;

{the timer serves as a mouse reset. It is here to avoid the screen saver
being deactivated by bumps on the table if the user has a high resolution mouse.
This constantly resets the 'IgnoreMouse' variable every 2 seconds.
The value of 'TicksToIgnore' controls how nice your saver behaves to these
unwanted mouse movements. You may want to change the time value or you may want
to disable or delete this timer as MS Screen Savers don't do stuff like this}
procedure TD97.MouseTimerTimer(Sender:TObject);
begin
     IgnoreMouse:=TicksToIgnore;
end;

procedure TD97.Min2FadeTimer(Sender:TObject);
begin //Increment number of min since last Fade to black
     Inc(NrMin);
end;

procedure TD97.FormClose(Sender:TObject; var Action:TCloseAction);
var Dummy:Integer;
begin
  if Assigned(MyThread) then
  begin
    MyThread.Terminate;
    MyThread:=NIL;
    SystemParametersInfo(97,Word(False),@Dummy,0); //enable every Win95 system key
    ShowCursor(True);
  end;
end;

procedure TD97.CreateParams (var Params:TCreateParams);
begin
  inherited CreateParams(Params);
  Params.ExStyle:=Params.ExStyle or WS_EX_TOPMOST; //make the window topmost
end;

procedure TD97.WMKeyDown;
begin
  CloseIfOk;
end;

procedure TD97.WMSysKeyDown;
begin
  CloseIfOk;
end;

procedure TD97.WMEraseBkGnd;
begin
  Msg.Result:=0; //never write to the background
end;

procedure TD97.MouseEvent;
begin //if the mouse is moved enough, close screensaver
  if IgnoreMouse>0 then
    Dec(IgnoreMouse)
  else
    CloseIfOK;
end;

//this function checks for password if necessary and closes
//the screensaver if pwd was correct
procedure TD97.CloseIfOk;
var hLib:THandle;
    P:function(Parent:THandle):Boolean; stdcall;
    SysDir:String;
    NewLength:Integer;
    Registry:TRegistry;
    OSVer:TOSVersionInfo;
begin //check if password must be checked
  Password:=True;
  OSVer.dwOSVersionInfoSize:=SizeOf(OSVer);
  GetVersionEx(OSVer);
  if OSVer.dwPlatformID=VER_PLATFORM_WIN32_WINDOWS then //If Windows 9x
  begin
       Registry:=TRegistry.Create;
       Registry.RootKey:=HKEY_CURRENT_USER;
       if Registry.OpenKey('Control Panel\desktop',false) then
       begin
            if Registry.ReadInteger('ScreenSaveUsePassword')<>0 then
            begin
                 ShowCursor(True); //activate mouse
                 //load library PASSWORD.CPL from system directory
                 SetLength(SysDir,MAX_PATH);
                 NewLength:=GetSystemDirectory(PChar(SysDir),MAX_PATH);
                 SetLength(SysDir,NewLength);
                 if (length(SysDir)>0) and (SysDir[length(SysDir)]<>'\') then
                    SysDir:=SysDir+'\';
                 hLib:=LoadLibrary(PChar(SysDir+'PASSWORD.CPL'));
                 if hLib=0 then
                    CloseSaver //if dll not found, simply close the saver
                 else
                 begin //show the password verification dialog
                       P:=GetProcAddress(hLib,'VerifyScreenSavePwd');
                       if P(Handle) then
                          CloseSaver;
                       FreeLibrary(hLib); //unload PASSWORD.CPL
                 end;
                 IgnoreMouse:=TicksToIgnore;
                 ShowCursor(False); //disable the mouse again
            end
            else
                CloseSaver;
       end
       else
           CloseSaver;
  end
  else
      if OSVer.dwPlatformID=VER_PLATFORM_WIN32_NT then //If Windows NT
      begin //password handled by NT, simply close screensaver
          CloseSaver;
      end
      else //Neither NT nor Win9x (what?), close screensaver
          CloseSaver;
  Password:=False;
end;

procedure TD97.CloseSaver;
var Dummy:Integer;
begin
     if Assigned(AviPlayReg) then
     begin //no need to free AviPlay/AviPanel since they are freed by their owner
       AviPlayReg.CloseKey;
       AviPlayReg.Free;
     end;
     MyThread.Terminate;
     MyThread:=NIL; //make sure it does not get terminated twice
     SystemParametersInfo(97,Word(False),@Dummy,0); //enable every Win95 system key
     ShowCursor(True);
     Close;
end;

//This executes the preview window,and goes into a message loop to keep it running
procedure ExecPreview;
var PreviewCanvas:TCanvas;
    WndClass:TWndClass;
    Atom:TAtom;
    DC:hDC;
    MyWnd:hWnd;
    Msg:TMsg;
begin //create a new window class
  PreviewMode:=True;
  with WndClass do begin
    style:=CS_PARENTDC;
    lpfnWndProc:=@MyWndProc;
    cbClsExtra:=0;
    cbWndExtra:=0;
    hIcon:=0;
    hCursor:=0;
    hbrBackground:=0;
    lpszMenuName:=NIL;
    lpszClassName:='Delphi2ScreenSaverPreview';
  end;
  WndClass.hInstance:=hInstance;
  Atom:=Windows.RegisterClass(WndClass);
  //get some info on parent window
  GetWindowRect(ParamHandle, PrevRect);
  PrevRect.Right:=PrevRect.Right-PrevRect.Left;
  PrevRect.Bottom:=PrevRect.Bottom-PrevRect.Top;
  PrevRect.Left:=0;
  PrevRect.Top:=0;
  //and now create the window as child of the window given in ParamHandle
  MyWnd:=CreateWindow('Delphi2ScreenSaverPreview','FunSaver',WS_CHILD or
     WS_DISABLED or WS_VISIBLE,0,0,PrevRect.Right,PrevRect.Bottom,
     ParamHandle,0,hInstance,nil);
  //get the DC for the new created window
  DC:=GetDC(MyWnd);
  PreviewCanvas:=TCanvas.Create;
  PreviewCanvas.Handle:=DC;
  MyThread:=TSaverThread.Create(PreviewCanvas,PrevRect);
  //and now do the message loop, I think I can't use any predifined loop here
  while GetMessage(Msg,0,0,0) do begin
    TranslateMessage(Msg);
    DispatchMessage(Msg);
  end;
  MyThread.Terminate; //I don't care about other resources here, as my app
  PreviewCanvas.Free; //terminates immediately and NT/Win95 cleans up
end;

procedure ExecSetPwd;
var hLib:THandle;
    P:function (a:PChar; ParentHandle:THandle; b,c:Integer):Integer; stdcall;
    SysDir:String;
    NewLength:Integer;
    OSVer:TOSVersionInfo;
begin
  OSVer.dwOSVersionInfoSize:=SizeOf(OSVer);
  GetVersionEx(OSVer);
  if OSVer.dwPlatformID=VER_PLATFORM_WIN32_WINDOWS then
  begin
       SetLength(SysDir,MAX_PATH);
       NewLength:=GetSystemDirectory (PChar(SysDir),MAX_PATH);
       SetLength(SysDir,NewLength);
       if (length(SysDir)>0) and (SysDir[length(SysDir)]<>'\') then
          SysDir:=SysDir+'\';
       hLib:=LoadLibrary(PChar(SysDir+'MPR.DLL'));
       if hLib<>0 then
       begin
            P:=GetProcAddress(hLib,'PwdChangePasswordA');
            if Assigned(P) then
               P('SCRSAVE',ParamHandle,0,0);
            FreeLibrary(hLib);
       end;
  end;
end;

Initialization
   FirstRun:=True;

end.
